package org.processmining.logprojection;

import org.processmining.logprojection.sorting.AlphanumComparator;

public class AttributeBasedObjectSorter {
	private int[] numbers;
	private DataManager<?> manager;
	private int attributeIndex;

	public void sort(DataManager<?> manager, int attributeIndex,
			byte attributeType, int[] numbers) {
		this.manager = manager;
		this.attributeIndex = attributeIndex;
		this.numbers = numbers;
		int number = numbers.length - 1;

		if (attributeType == Attribute.BOOL) {
			// TODO?
		} else if (attributeType == Attribute.LITR) {
			quicksortLiteral(0, number);
		} else if (attributeType == Attribute.CONT) {
			quicksortContinuous(0, number);
		} else if (attributeType == Attribute.DISC) {
			quicksortDiscrete(0, number);
		} else if (attributeType == Attribute.TIME_ABS
				|| attributeType == Attribute.TIME_REL) {
			quicksortTimestamp(0, number);
		}
	}

	private void quicksortLiteral(int low, int high) {
		AlphanumComparator comparator = new AlphanumComparator();

		int i = low, j = high;
		// Get the pivot element from the middle of the list
		String pivot = manager.getAttributeValueLiteral(numbers[low
				+ (high - low) / 2], attributeIndex);

		// Divide into two lists
		while (i <= j) {
			// If the current value from the left list is smaller then the
			// pivot element then get the next element from the left list
			 while (comparator.compare(manager.getAttributeValueLiteral(
			 numbers[i], attributeIndex), pivot) < 0) {
				// while
				// (pivot.compareTo(manager.getAttributeValueLiteral(numbers[i],
				// attributeIndex)) < 0) {
				i++;
			}
			// If the current value from the right list is larger then the
			// pivot element then get the next element from the right list
			 while (comparator.compare(manager.getAttributeValueLiteral(
			 numbers[j], attributeIndex), pivot) > 0) {
//			while (pivot.compareTo(manager.getAttributeValueLiteral(numbers[j],
//					attributeIndex)) > 0) {
				j--;
			}

			// If we have found a values in the left list which is larger
			// then the pivot element and if we have found a value in the right
			// list which is smaller then the pivot element then we exchange the
			// values. As we are done we can increase i and j
			if (i <= j) {
				exchange(i, j);
				i++;
				j--;
			}
		}
		// Recursion
		if (low < j)
			quicksortLiteral(low, j);
		if (i < high)
			quicksortLiteral(i, high);
	}

	private void quicksortContinuous(int low, int high) {
		int i = low, j = high;
		// Get the pivot element from the middle of the list
		double pivot = manager.getAttributeValueContinuous(numbers[low
				+ (high - low) / 2], attributeIndex);

		// Divide into two lists
		while (i <= j) {
			// If the current value from the left list is smaller then the
			// pivot element then get the next element from the left list
			while (manager.getAttributeValueContinuous(numbers[i],
					attributeIndex) < pivot) {
				i++;
			}
			// If the current value from the right list is larger then the
			// pivot element then get the next element from the right list
			while (manager.getAttributeValueContinuous(numbers[j],
					attributeIndex) > pivot) {
				j--;
			}

			// If we have found a values in the left list which is larger
			// then the pivot element and if we have found a value in the right
			// list which is smaller then the pivot element then we exchange the
			// values. As we are done we can increase i and j
			if (i <= j) {
				exchange(i, j);
				i++;
				j--;
			}
		}
		// Recursion
		if (low < j)
			quicksortContinuous(low, j);
		if (i < high)
			quicksortContinuous(i, high);
	}

	private void quicksortDiscrete(int low, int high) {
		int i = low, j = high;
		// Get the pivot element from the middle of the list
		long pivot = manager.getAttributeValueDiscrete(numbers[low
				+ (high - low) / 2], attributeIndex);

		// Divide into two lists
		while (i <= j) {
			// If the current value from the left list is smaller then the
			// pivot element then get the next element from the left list
			while (manager
					.getAttributeValueDiscrete(numbers[i], attributeIndex) < pivot) {
				i++;
			}
			// If the current value from the right list is larger then the
			// pivot element then get the next element from the right list
			while (manager
					.getAttributeValueDiscrete(numbers[j], attributeIndex) > pivot) {
				j--;
			}

			// If we have found a values in the left list which is larger
			// then the pivot element and if we have found a value in the right
			// list which is smaller then the pivot element then we exchange the
			// values. As we are done we can increase i and j
			if (i <= j) {
				exchange(i, j);
				i++;
				j--;
			}
		}
		// Recursion
		if (low < j)
			quicksortDiscrete(low, j);
		if (i < high)
			quicksortDiscrete(i, high);
	}

	private void quicksortTimestamp(int low, int high) {
		int i = low, j = high;
		// Get the pivot element from the middle of the list
		long pivot = manager.getAttributeValueDiscrete(numbers[low
				+ (high - low) / 2], attributeIndex);

		// Divide into two lists
		while (i <= j) {
			// If the current value from the left list is smaller then the
			// pivot element then get the next element from the left list
			while (manager.getAttributeValueTimestamp(numbers[i],
					attributeIndex) < pivot) {
				i++;
			}
			// If the current value from the right list is larger then the
			// pivot element then get the next element from the right list
			while (manager.getAttributeValueTimestamp(numbers[j],
					attributeIndex) > pivot) {
				j--;
			}

			// If we have found a values in the left list which is larger
			// then the pivot element and if we have found a value in the right
			// list which is smaller then the pivot element then we exchange the
			// values. As we are done we can increase i and j
			if (i <= j) {
				exchange(i, j);
				i++;
				j--;
			}
		}
		// Recursion
		if (low < j)
			quicksortTimestamp(low, j);
		if (i < high)
			quicksortTimestamp(i, high);
	}

	private void exchange(int i, int j) {
		int temp = numbers[i];
		numbers[i] = numbers[j];
		numbers[j] = temp;
	}
}