package org.processmining.logprojection;

public interface Histogram {

	public static class Double implements Histogram {

		private int[] buckets;
		private Statistics.Double s;
		private double scale;

		public Double(Statistics.Double s, int buckets) {

			this.buckets = new int[buckets];
			this.s = s;
			this.scale = s.getMaximum() - s.getMinimum();
		}

		public void addObservation(int value) {
			addObservation((double) value);
		}

		public void addObservation(short value) {
			addObservation((double) value);
		}

		public void addObservation(float value) {
			addObservation((double) value);
		}

		public void addObservation(long value) {
			addObservation((double) value);
		}

		public void addObservation(double value) {
			if (scale > 0) {
				int bucket = (int) (((value - s.getMinimum()) / scale) * buckets.length);
				if (bucket == buckets.length) {
					bucket--;
				}
				buckets[bucket]++;
			}
		}

		public int getBucketFor(int value) {
			return getBucketFor((double) value);
		}

		public int getBucketFor(short value) {
			return getBucketFor((double) value);
		}

		public int getBucketFor(float value) {
			return getBucketFor((double) value);
		}

		public int getBucketFor(double value) {
			int bucket = (int) (((value - s.getMinimum()) / scale) * buckets.length);
			if (bucket == buckets.length) {
				bucket--;
			}
			return bucket;
		}

		public int getBucketFor(long value) {
			return getBucketFor((double) value);
		}

		public double getBucketWidth() {
			return scale / buckets.length;
		}

		public int[] getHistogram() {
			return buckets;
		}
	}

	public static class Long implements Histogram {

		private int[] buckets;
		private Statistics.Long s;
		private double scale;

		public Long(Statistics.Long s, int buckets) {

			this.buckets = new int[buckets];
			this.s = s;
			this.scale = s.getMaximum() - s.getMinimum();
		}

		public void addObservation(int value) {
			addObservation((long) value);
		}

		public void addObservation(short value) {
			addObservation((long) value);
		}

		public void addObservation(float value) {
			addObservation((long) value);
		}

		public void addObservation(long value) {
			if (scale > 0) {
				int bucket = (int) (((value - s.getMinimum()) / scale) * buckets.length);
				if (bucket == buckets.length) {
					bucket--;
				}
				buckets[bucket]++;
			}
		}

		public void addObservation(double value) {
			addObservation((long) value);
		}

		public int getBucketFor(int value) {
			return getBucketFor((long) value);
		}

		public int getBucketFor(short value) {
			return getBucketFor((long) value);
		}

		public int getBucketFor(float value) {
			return getBucketFor((long) value);
		}

		public int getBucketFor(long value) {
			int bucket = (int) (((value - s.getMinimum()) / scale) * buckets.length);
			if (bucket == buckets.length) {
				bucket--;
			}
			return bucket;
		}

		public int getBucketFor(double value) {
			return getBucketFor((long) value);
		}

		public int[] getHistogram() {
			return buckets;
		}

		public double getBucketWidth() {
			return scale / buckets.length;
		}

	}

	public void addObservation(int value);

	public void addObservation(short value);

	public void addObservation(float value);

	public void addObservation(long value);

	public void addObservation(double value);

	public int[] getHistogram();

	public double getBucketWidth();

	public int getBucketFor(int value);

	public int getBucketFor(short value);

	public int getBucketFor(float value);

	public int getBucketFor(long value);

	public int getBucketFor(double value);

}
