package org.processmining.logprojection.calculators;

import java.util.Arrays;

import org.deckfour.xes.model.XAttributable;
import org.deckfour.xes.model.XAttribute;
import org.processmining.logprojection.Attribute;
import org.processmining.logprojection.DataManager;
import org.processmining.logprojection.DataManager.AttributeValueCalculator;
import org.processmining.logprojection.LogView;

public abstract class AbstractCategoryCalculator<T extends XAttributable> implements AttributeValueCalculator<T> {

	public static class ContinuousCategoryCalculator<T extends XAttributable> extends AbstractCategoryCalculator<T> {

		final private double[] categories;

		public ContinuousCategoryCalculator(LogView view, Attribute continuousAttribute, boolean event) {
			super(view, continuousAttribute, event);

			double[] values = dataManager.getAttributeValuesContinuous(continuousAttribute, true);

			categories = new double[Math.min(values.length, 8)];

			// chop in 8 equal parts
			int part = values.length / categories.length;
			for (int i = 0; i < categories.length; i++) {
				categories[i] = values[i * part];
				names[i] = Double.toString(categories[i]);
			}
		}

		public XAttribute[] computeAttributes(T object, int objectIndex, int... header) {

			int v = Arrays.binarySearch(categories,
					dataManager.getAttributeValueContinuous(objectIndex, originalAttribute));
			if (v < 0) {
				v = -v - 2;
			}
			XAttribute a1 = new XLightLiteralAttribute(key, names[v]);
			return new XAttribute[] { a1 };

		}

	}

	public static class DiscreteCategoryCalculator<T extends XAttributable> extends AbstractCategoryCalculator<T> {

		final private long[] categories;

		public DiscreteCategoryCalculator(LogView view, Attribute discreteAttribute, boolean event) {
			super(view, discreteAttribute, event);

			long[] values = dataManager.getAttributeValuesDiscrete(discreteAttribute, true);

			categories = new long[Math.min(values.length, 8)];

			// chop in 8 equal parts
			int part = values.length / categories.length;
			for (int i = 0; i < categories.length; i++) {
				categories[i] = values[i * part];
				names[i] = Long.toString(categories[i]);
			}

		}

		public XAttribute[] computeAttributes(T object, int objectIndex, int... header) {

			int v = Arrays.binarySearch(categories,
					dataManager.getAttributeValueDiscrete(objectIndex, originalAttribute));
			if (v < 0) {
				v = -v - 2;
			}
			XAttribute a1 = new XLightLiteralAttribute(key, names[v]);
			return new XAttribute[] { a1 };

		}

	}

	final protected String key;
	final protected Attribute originalAttribute;
	final protected Attribute resultAttribute;
	final protected DataManager<?> dataManager;
	final protected String[] names;

	private AbstractCategoryCalculator(LogView view, Attribute originalAttribute, boolean isEventLevel) {
		this.originalAttribute = originalAttribute;
		this.dataManager = isEventLevel ? view.getEventDataManager() : view.getTraceDataManager();
		key = originalAttribute.getKey() + " Category";
		names = new String[8];
		resultAttribute = new Attribute(key, Attribute.LITR, 0, isEventLevel);
	}

	public Attribute getAttributes() {
		return resultAttribute;
	}

}
