package org.processmining.logprojection.filters;

import org.processmining.logprojection.Attribute;
import org.processmining.logprojection.LogView;

public final class BooleanAttributeFilter extends AbstractEventFilter {
	private final boolean keepTrue;
	private final boolean keepFalse;

	/**
	 * keeps events if the attribute value is true if keepTrue is set and if the
	 * value is false if keepFalse is set.
	 * 
	 * @param view
	 * @param attribute
	 *            The attribute to be checked for each event. Note that this has
	 *            to be a boolean attribute, i.e. the type should be
	 *            Attribute.BOOL, otherwise results are unpredictable
	 * @param isEventAttribute
	 * @param min
	 * @param max
	 * @param inverted
	 */
	public BooleanAttributeFilter(LogView view, Attribute attribute,
			boolean keepTrue, boolean keepFalse) {
		super(view, attribute);
		this.keepTrue = keepTrue;
		this.keepFalse = keepFalse;
	}

	@Override
	public boolean keepEvent(LogView view, int traceIndex, int eventIndex) {
		boolean val;
		if (attribute.isEventAttribute()) {
			val = getValue(eventIndex);
		} else {
			val = getValue(traceIndex);
		}
		return (val && keepTrue) || (!val && keepFalse);
	}

	private boolean getValue(int objectIndex) {
		switch (attribute.getType()) {
		case Attribute.BOOL:
			return manager
					.getAttributeValueBoolean(objectIndex, attributeIndex);
		default:
			return keepTrue;
		}
	}

}